#!/usr/bin/perl -w

# netlogon.pl v0.1

# Copyright (c) 2002, Deryk Robosson
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:

#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#    * Neither the name of the Robosson Business Services nor the
#      names of its contributors may be used to endorse or promote
#      products derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
use strict;
use XML::Twig;

my $user_name = $ARGV[0] || '';
my $group_name = $ARGV[1] || '';
my $arch_name = $ARGV[2] || '';
my $net_name = $ARGV[3] || '';


# Global variables used throughout the script
my ($maindrive, $userdrive, $servername);
my (@groups, @users);
my $file = 'netlogon.xml';
my $output_path = './';
my $script = "\@ECHO \"Configuring network services...\"\r\n";
my $DEBUG = 0; # Set to 1 to enable debug output


# XML::Twig handlers
my $twig_handlers = {'global' => \&global_handler,
		     'group' => \&group_handler,
		     'user' => \&user_handler};
my $twig = new XML::Twig(TwigHandlers => $twig_handlers);


$twig->parsefile($file);

# Set the time, mount the global share and users home share
$script .= "\@ECHO \"Setting time...\"\r\n";
$script .= "NET TIME \\\\$servername /SET /YES\r\n";
$script .= "\@ECHO \"Mounting $maindrive...\"\r\n";
$script .= "NET USE $maindrive \\\\$servername\\space /YES\r\n";
$script .= "\@ECHO \"Mounting $userdrive...\"\r\n";
$script .= "NET USE $userdrive \\\\$servername\\$user_name /YES\r\n";


if($DEBUG == 1) {
    print "Main: $maindrive\n";
    print "User: $userdrive\n";
    print "Server: $servername\n";
}


# Iterate through the groups
foreach my $group (@groups) {
    if($group_name eq $group->{name}) {
	$script .= "\@ECHO \"".$group->{comment}."\"\r\n";
	$script .= "NET USE ".$group->{drive}." \\\\".$servername."\\".$group->{share}." /YES\r\n";
    }
    if($DEBUG == 1) {
	print "Group->\n";
	print "\tName: ".$group->{name}."\n";
	print "\tComment: ".$group->{comment}."\n";
	print "\tShare: ".$group->{share}."\n";
    }
}


# Iterate through the users
foreach my $user (@users) {
    if($user_name eq $user->{name}) {
	$script .= "\@ECHO \"".$user->{comment}."\"\r\n";
	$script .= "NET USE ".$user->{drive}." \\\\".$servername."\\".$user->{share}." /YES\r\n";
    }
    if($DEBUG == 1) {
	print "User->\n";
	print "\tName: ".$user->{name}."\n";
	print "\tComment: ".$user->{comment}."\n";
	print "\tShare: ".$user->{share}."\n";
    }
}


if($DEBUG == 1) {
    print "\n\n".$script."\n\n";
}
else {
# Dump the final script to a file
    my $filename = $output_path . "$user_name.bat";
    if(open(SCRIPT, ">$filename")) {
	print SCRIPT $script;
	close(SCRIPT);
    }
}


# Handle the <global/> tag(s)
sub global_handler
{
    my ($twig_obj, $global) = @_;
    $maindrive = $global->first_child('maindrive')->text;
    $userdrive = $global->first_child('userdrive')->text;
    $servername = $global->first_child('servername')->text;
}


# Handle the <group/> tag(s)
sub group_handler
{
    my ($twig_obj, $group) = @_;
    my $grp;
    $grp->{name} = $group->first_child('name')->text;
    $grp->{comment} = $group->first_child('comment')->text;
    $grp->{share} = $group->first_child('share')->text;
    $grp->{drive} = $group->first_child('drive')->text;
    push @groups, $grp;
}


# Handle the <user/> tag(s)
sub user_handler
{
    my($twig_obj, $user) = @_;
    my $usr;
    $usr->{name} = $user->first_child('name')->text;
    $usr->{comment} = $user->first_child('comment')->text;
    $usr->{share} = $user->first_child('share')->text;
    $usr->{drive} = $user->first_child('drive')->text;
    push @users, $usr;
}
